package sun.security.provider;

import java.security.ProviderException;
import java.security.MessageDigest;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidParameterSpecException;
import java.security.spec.DSAParameterSpec;
import java.security.AlgorithmParameters;
import java.security.InvalidAlgorithmParameterException;
import java.security.spec.DSAGenParameterSpec;
import java.security.spec.AlgorithmParameterSpec;
import java.security.InvalidParameterException;
import sun.security.util.SecurityProviderConstants;
import java.math.BigInteger;
import java.security.SecureRandom;
import java.security.AlgorithmParameterGeneratorSpi;

public class DSAParameterGenerator extends AlgorithmParameterGeneratorSpi
{
    private int valueL;
    private int valueN;
    private int seedLen;
    private SecureRandom random;
    private static final BigInteger TWO;
    
    public DSAParameterGenerator() {
        this.valueL = -1;
        this.valueN = -1;
        this.seedLen = -1;
    }
    
    @Override
    protected void engineInit(int i, SecureRandom random) {
        if (i != 2048 && i != 3072 && (i < 512 || i > 1024 || i % 64 != 0)) {
            throw new InvalidParameterException("Unexpected strength (size of prime): " + i + ". Prime size should be 512-1024, 2048, or 3072");
        }
        this.valueL = i;
        this.valueN = SecurityProviderConstants.getDefDSASubprimeSize(i);
        this.seedLen = this.valueN;
        this.random = random;
        return;
    }
    
    @Override
    protected void engineInit(AlgorithmParameterSpec algorithmParameterSpec, SecureRandom random) throws InvalidAlgorithmParameterException {
        if (!(algorithmParameterSpec instanceof DSAGenParameterSpec)) {
            throw new InvalidAlgorithmParameterException("Invalid parameter");
        }
        DSAGenParameterSpec dsaGenParameterSpec = (DSAGenParameterSpec)algorithmParameterSpec;
        this.valueL = dsaGenParameterSpec.getPrimePLength();
        this.valueN = dsaGenParameterSpec.getSubprimeQLength();
        this.seedLen = dsaGenParameterSpec.getSeedLength();
        this.random = random;
        return;
    }
    
    @Override
    protected AlgorithmParameters engineGenerateParameters() {
        AlgorithmParameters instance;
        try {
            if (this.random == null) {
                this.random = new SecureRandom();
            }
            if (this.valueL == -1) {
                this.engineInit(SecurityProviderConstants.DEF_DSA_KEY_SIZE, this.random);
            }
            BigInteger[] generatePandQ = generatePandQ(this.random, this.valueL, this.valueN, this.seedLen);
            BigInteger p = generatePandQ[0];
            BigInteger q = generatePandQ[1];
            DSAParameterSpec paramSpec = new DSAParameterSpec(p, q, generateG(p, q));
            instance = AlgorithmParameters.getInstance("DSA", "SUN");
            instance.init(paramSpec);
        }
        catch (InvalidParameterSpecException ex) {
            throw new RuntimeException(ex.getMessage());
        }
        catch (NoSuchAlgorithmException ex2) {
            throw new RuntimeException(ex2.getMessage());
        }
        catch (NoSuchProviderException ex3) {
            throw new RuntimeException(ex3.getMessage());
        }
        return instance;
    }
    
    private static BigInteger[] generatePandQ(SecureRandom secureRandom, int i, int n, int exponent) {
        String algorithm = null;
        if (n == 160) {
            algorithm = "SHA";
        }
        else if (n == 224) {
            algorithm = "SHA-224";
        }
        else if (n == 256) {
            algorithm = "SHA-256";
        }
        MessageDigest instance = null;
        try {
            instance = MessageDigest.getInstance(algorithm);
        }
        catch (NoSuchAlgorithmException ex) {
            ex.printStackTrace();
        }
        int n2 = instance.getDigestLength() * 8;
        int n3 = (i - 1) / n2;
        int exponent2 = (i - 1) % n2;
        byte[] input = new byte[exponent / 8];
        BigInteger pow = DSAParameterGenerator.TWO.pow(exponent);
        int n4 = -1;
        if (i <= 1024) {
            n4 = 80;
        }
        else if (i == 2048) {
            n4 = 112;
        }
        else if (i == 3072) {
            n4 = 128;
        }
        if (n4 < 0) {
            throw new ProviderException("Invalid valueL: " + i);
        }
        Object o;
        BigInteger subtract;
        int j;
        BigInteger subtract2;
    Block_14:
        while (true) {
            secureRandom.nextBytes(input);
            o = new BigInteger(1, input);
            BigInteger mod = new BigInteger(1, instance.digest(input)).mod(DSAParameterGenerator.TWO.pow(n - 1));
            subtract = DSAParameterGenerator.TWO.pow(n - 1).add(mod).add(BigInteger.ONE).subtract(mod.mod(DSAParameterGenerator.TWO));
            if (!subtract.isProbablePrime(n4)) {
                continue;
            }
            BigInteger val = BigInteger.ONE;
            j = 0;
            while (j < 4 * i) {
                BigInteger[] array = new BigInteger[n3 + 1];
                for (int k = 0; k <= n3; ++k) {
                    array[k] = new BigInteger(1, instance.digest(toByteArray(o.add(val).add(BigInteger.valueOf((long)k)).mod(pow))));
                }
                BigInteger add = array[0];
                for (int l = 1; l < n3; ++l) {
                    add = add.add(array[l].multiply(DSAParameterGenerator.TWO.pow(l * n2)));
                }
                BigInteger add2 = add.add(array[n3].mod(DSAParameterGenerator.TWO.pow(exponent2)).multiply(DSAParameterGenerator.TWO.pow(n3 * n2)));
                BigInteger pow2 = DSAParameterGenerator.TWO.pow(i - 1);
                BigInteger add3 = add2.add(pow2);
                subtract2 = add3.subtract(add3.mod(subtract.multiply(DSAParameterGenerator.TWO)).subtract(BigInteger.ONE));
                if (subtract2.compareTo(pow2) > -1 && subtract2.isProbablePrime(n4)) {
                    break Block_14;
                }
                val = val.add(BigInteger.valueOf((long)n3)).add(BigInteger.ONE);
                ++j;
            }
        }
        return new BigInteger[] { subtract2, subtract, o, BigInteger.valueOf((long)j) };
    }
    
    private static BigInteger generateG(BigInteger m, BigInteger val) {
        BigInteger bigInteger;
        BigInteger divide;
        BigInteger bigInteger2;
        for (bigInteger = BigInteger.ONE, divide = m.subtract(BigInteger.ONE).divide(val), bigInteger2 = BigInteger.ONE; bigInteger2.compareTo(DSAParameterGenerator.TWO) < 0; bigInteger2 = bigInteger.modPow(divide, m), bigInteger = bigInteger.add(BigInteger.ONE)) {}
        return bigInteger2;
    }
    
    private static byte[] toByteArray(BigInteger bigInteger) {
        byte[] byteArray = bigInteger.toByteArray();
        if (byteArray[0] == 0) {
            byte[] array = new byte[byteArray.length - 1];
            System.arraycopy(byteArray, 1, array, 0, array.length);
            byteArray = array;
            return byteArray;
        }
        return byteArray;
    }
    
    static {
        TWO = BigInteger.valueOf(2L);
    }
}
